@@ -8,22 +8,22 @@ from neosctl.util import process_response
 app = typer.Typer()
 
 
-def consume_url(gateway_api_url: str) -> str:
-    return "{}/consume".format(gateway_api_url.rstrip("/"))
+def _consume_url(ctx: typer.Context) -> str:
+    return "{}/consume".format(ctx.obj.gateway_api_url.rstrip("/"))
 
 
 @app.command()
 def query(
     ctx: typer.Context,
-    statement: str,
-):
+    statement: str = typer.Argument(..., callback=util.sanitize),
+) -> None:
     """Query a published data product."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            url=consume_url(ctx.obj.gateway_api_url),
+            url=_consume_url(ctx),
             json={
                 "statement": statement,
             },
