@@ -20,22 +20,26 @@ app.add_typer(browse_app, name="browse", help="Browse external data source metad
 app.add_typer(pipeline_app, name="pipeline", help="Manage data source pipelines.")
 
 
-def metadata_url(ctx: typer.Context, postfix: str = "") -> str:
+def _metadata_url(ctx: typer.Context, postfix: str = "") -> str:
     return "{}/metadata{}".format(ctx.obj.get_gateway_api_url().rstrip("/"), postfix)
 
 
 @product_app.command()
 def get(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get data product metadata."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}"),
+            _metadata_url(ctx, f"/product/{product_name}"),
         )
 
     r = _request(ctx)
@@ -46,16 +50,16 @@ def get(
 @tag_app.command("add")
 def add_tag(
     ctx: typer.Context,
-    tag: str,
-    scope: str = typer.Option("field", "--scope", "-s"),
-):
+    tag: str = typer.Argument(..., callback=util.sanitize),
+    scope: str = typer.Option("field", "--scope", "-s", callback=util.sanitize),
+) -> None:
     """Add a tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, "/tag"),
+            _metadata_url(ctx, "/tag"),
             json={
                 "tag": tag.lower(),
                 "scope": scope.upper(),
@@ -70,16 +74,16 @@ def add_tag(
 @tag_app.command("remove")
 def remove_tag(
     ctx: typer.Context,
-    tag: str,
-    scope: str = typer.Option("field", "--scope", "-s"),
-):
+    tag: str = typer.Argument(..., callback=util.sanitize),
+    scope: str = typer.Option("field", "--scope", "-s", callback=util.sanitize),
+) -> None:
     """Remove a tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            url=metadata_url(ctx, "/tag"),
+            url=_metadata_url(ctx, "/tag"),
             json={
                 "tag": tag.lower(),
                 "scope": scope.upper(),
@@ -94,10 +98,11 @@ def remove_tag(
 @tag_app.command("list")
 def list_tags(
     ctx: typer.Context,
-    tag_filter: str = typer.Option(None, "--tag-filter", "-t"),
-    scope: str = typer.Option("field", "--scope", "-s"),
+    *,
+    tag_filter: str = typer.Option(None, "--tag-filter", "-t", callback=util.sanitize),
+    scope: str = typer.Option("field", "--scope", "-s", callback=util.sanitize),
     system_defined: bool = typer.Option(False, "--system-defined"),
-):
+) -> None:
     """List tags.
 
     Tag list can be filtered using `-t/--tag-filter`, and the tag scope can be
@@ -111,7 +116,7 @@ def list_tags(
             params["tag_filter"] = tag_filter
         return util.get(
             ctx,
-            url=metadata_url(ctx, "/tag"),
+            url=_metadata_url(ctx, "/tag"),
             params=params,
         )
 
@@ -121,14 +126,14 @@ def list_tags(
 
 
 @browse_app.command()
-def source_types(ctx: typer.Context):
+def source_types(ctx: typer.Context) -> None:
     """List source types."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            url=metadata_url(ctx, "/browse"),
+            url=_metadata_url(ctx, "/browse"),
         )
 
     r = _request(ctx)
@@ -139,15 +144,15 @@ def source_types(ctx: typer.Context):
 @browse_app.command("databases")
 def browse_databases(
     ctx: typer.Context,
-    source_type: str = typer.Option(..., "--source-type", "-s"),
-):
+    source_type: str = typer.Option(..., "--source-type", "-s", callback=util.sanitize),
+) -> None:
     """List databases by source type."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            url=metadata_url(ctx, f"/browse/{source_type}"),
+            url=_metadata_url(ctx, f"/browse/{source_type}"),
         )
 
     r = _request(ctx)
@@ -158,16 +163,16 @@ def browse_databases(
 @browse_app.command("datasets")
 def browse_datasets(
     ctx: typer.Context,
-    source_type: str = typer.Option(..., "--source-type", "-s"),
-    database: str = typer.Option(..., "--database", "-d"),
-):
+    source_type: str = typer.Option(..., "--source-type", "-s", callback=util.sanitize),
+    database: str = typer.Option(..., "--database", "-d", callback=util.sanitize),
+) -> None:
     """List datasets by source type and database."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            url=metadata_url(ctx, f"/browse/{source_type}/{database}"),
+            url=_metadata_url(ctx, f"/browse/{source_type}/{database}"),
         )
 
     r = _request(ctx)
@@ -178,17 +183,17 @@ def browse_datasets(
 @browse_app.command("dataset-metadata")
 def browse_dataset_metadata(
     ctx: typer.Context,
-    source_type: str = typer.Option(..., "--source-type", "-s"),
-    database: str = typer.Option(..., "--database", "-d"),
-    dataset_urn: str = typer.Option(..., "--dataset-urn", "-du"),
-):
+    source_type: str = typer.Option(..., "--source-type", "-s", callback=util.sanitize),
+    database: str = typer.Option(..., "--database", "-d", callback=util.sanitize),
+    dataset_urn: str = typer.Option(..., "--dataset-urn", "-du", callback=util.sanitize),
+) -> None:
     """Get dataset metadata by source type, database and dataset urn."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            url=metadata_url(ctx, f"/browse/{source_type}/{database}/{dataset_urn}"),
+            url=_metadata_url(ctx, f"/browse/{source_type}/{database}/{dataset_urn}"),
         )
 
     r = _request(ctx)
@@ -199,23 +204,26 @@ def browse_dataset_metadata(
 @pipeline_app.command("add")
 def add_pipeline(
     ctx: typer.Context,
-    name: str = typer.Option(..., "--name", "-n"),
+    name: str = typer.Option(..., "--name", "-n", callback=util.sanitize),
+    *,
     schedule: str = typer.Option(
         ...,
         "--schedule",
         "-s",
         help='Pipeline schedule in crontab format (e.g. "* * * * *")',
+        callback=util.sanitize,
     ),
-    schedule_timezone: str = typer.Option(..., "--schedule-timezone", "-st"),
+    schedule_timezone: str = typer.Option(..., "--schedule-timezone", "-st", callback=util.sanitize),
     source: str = typer.Option(
         ...,
         "--source",
         "-sr",
         help="Data source DSN (e.g. mysql://user:pass@host:3306/dbname)",
+        callback=util.sanitize,
     ),
     include_tables: bool = typer.Option(True, "--include-tables", "-it", help="Applies only to relational DBs"),
     include_views: bool = typer.Option(True, "--include-views", "-iv", help="Applies only to relational DBs"),
-):
+) -> None:
     """Add an ingestion pipeline."""
     source_url = httpx.URL(source)
     request_data = {
@@ -237,7 +245,7 @@ def add_pipeline(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, "/pipeline"),
+            _metadata_url(ctx, "/pipeline"),
             json=request_data,
         )
 
@@ -249,15 +257,15 @@ def add_pipeline(
 @pipeline_app.command("remove")
 def remove_pipeline(
     ctx: typer.Context,
-    name: str = typer.Option(..., "--name", "-n"),
-):
+    name: str = typer.Option(..., "--name", "-n", callback=util.sanitize),
+) -> None:
     """Remove an ingestion pipeline."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            metadata_url(ctx, "/pipeline"),
+            _metadata_url(ctx, "/pipeline"),
             json={
                 "pipeline_name": name,
             },
@@ -271,16 +279,20 @@ def remove_pipeline(
 @product_app.command("description")
 def product_description(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    description: str = typer.Option(..., "--description", "-d"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    description: str = typer.Option(..., "--description", "-d", callback=util.sanitize),
+) -> None:
     """Add a product description."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/description"),
+            _metadata_url(ctx, f"/product/{product_name}/description"),
             json={
                 "description": description,
             },
@@ -294,16 +306,20 @@ def product_description(
 @product_app.command("tag")
 def product_tag(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    tag: str = typer.Option(..., "--tag", "-t"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    tag: str = typer.Option(..., "--tag", "-t", callback=util.sanitize),
+) -> None:
     """Add a product tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/tag"),
+            _metadata_url(ctx, f"/product/{product_name}/tag"),
             json={
                 "tag": tag.lower(),
             },
@@ -317,16 +333,20 @@ def product_tag(
 @product_app.command("remove-tag")
 def remove_product_tag(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    tag: str = typer.Option(..., "--tag", "-t"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    tag: str = typer.Option(..., "--tag", "-t", callback=util.sanitize),
+) -> None:
     """Remove a product tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            url=metadata_url(ctx, f"/product/{product_name}/tag"),
+            url=_metadata_url(ctx, f"/product/{product_name}/tag"),
             json={
                 "tag": tag.lower(),
             },
@@ -340,17 +360,21 @@ def remove_product_tag(
 @product_app.command("field-tag")
 def product_field_tag(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    field: str = typer.Option(..., "--field", "-f"),
-    tag: str = typer.Option(..., "--tag", "-t"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    field: str = typer.Option(..., "--field", "-f", callback=util.sanitize),
+    tag: str = typer.Option(..., "--tag", "-t", callback=util.sanitize),
+) -> None:
     """Add a product field tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
+            _metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
             json={
                 "tag": tag.lower(),
             },
@@ -364,17 +388,21 @@ def product_field_tag(
 @product_app.command("remove-field-tag")
 def remove_product_field_tag(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    field: str = typer.Option(..., "--field", "-f"),
-    tag: str = typer.Option(..., "--tag", "-t"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    field: str = typer.Option(..., "--field", "-f", callback=util.sanitize),
+    tag: str = typer.Option(..., "--tag", "-t", callback=util.sanitize),
+) -> None:
     """Remove a product field tag."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            url=metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
+            url=_metadata_url(ctx, f"/product/{product_name}/{field}/tag"),
             json={
                 "tag": tag.lower(),
             },
@@ -388,17 +416,21 @@ def remove_product_field_tag(
 @product_app.command("field-description")
 def product_field_description(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    field: str = typer.Option(..., "--field", "-f"),
-    description: str = typer.Option(..., "--description", "-d"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    field: str = typer.Option(..., "--field", "-f", callback=util.sanitize),
+    description: str = typer.Option(..., "--description", "-d", callback=util.sanitize),
+) -> None:
     """Add a product field description."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/{field}/description"),
+            _metadata_url(ctx, f"/product/{product_name}/{field}/description"),
             json={
                 "description": description,
             },
@@ -412,15 +444,19 @@ def product_field_description(
 @product_app.command("quality-expectations")
 def product_quality_expectations(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get data product quality expectations."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/quality/expectations"),
+            _metadata_url(ctx, f"/product/{product_name}/quality/expectations"),
         )
 
     r = _request(ctx)
@@ -431,15 +467,19 @@ def product_quality_expectations(
 @product_app.command("quality-profiling")
 def product_quality_profiling(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get data product quality profiling."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/quality/profiling"),
+            _metadata_url(ctx, f"/product/{product_name}/quality/profiling"),
         )
 
     r = _request(ctx)
@@ -450,15 +490,19 @@ def product_quality_profiling(
 @product_app.command("quality-validations")
 def product_quality_validations(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get data product quality validations."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            metadata_url(ctx, f"/product/{product_name}/quality/validations"),
+            _metadata_url(ctx, f"/product/{product_name}/quality/validations"),
         )
 
     r = _request(ctx)
