@@ -13,20 +13,24 @@ secret_app = typer.Typer()
 app.add_typer(secret_app, name="secret", help="Manage secrets for a spark job.")
 
 
-def spark_url(name: str, gateway_api_url: str) -> str:
+def _spark_url(name: str, gateway_api_url: str) -> str:
     return "{}/spark/{}".format(gateway_api_url.rstrip("/"), name)
 
 
-def secret_url(name: str, gateway_api_url: str) -> str:
+def _secret_url(name: str, gateway_api_url: str) -> str:
     return "{}/secret/{}".format(gateway_api_url.rstrip("/"), name)
 
 
 @app.command()
 def add_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    job_filepath: str = typer.Option(..., "--job-filepath", "-f"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    job_filepath: str = typer.Option(..., "--job-filepath", "-f", callback=util.sanitize),
+) -> None:
     """Assign a spark job.
 
     Assign and configure a spark job for a data product. This will result in a
@@ -37,7 +41,7 @@ def add_job(
     def _request(ctx: typer.Context, f: typing.IO) -> httpx.Response:
         return util.post(
             ctx,
-            spark_url(product_name, ctx.obj.gateway_api_url),
+            _spark_url(product_name, ctx.obj.gateway_api_url),
             files={"spark_file": f},
         )
 
@@ -52,8 +56,13 @@ def add_job(
 @app.command()
 def generate_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    json_filepath: str = typer.Option(..., "--json-filepath", "-f"),
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    json_filepath: str = typer.Option(..., "--json-filepath", "-f", callback=util.sanitize),
+    *,
     dry_run: bool = typer.Option(
         False,
         "--dry-run",
@@ -64,7 +73,7 @@ def generate_job(
         "--reassign",
         help="Generate spark code and replace existing spark job.",
     ),
-):
+) -> None:
     """Generate and assign a spark job.
 
     Generate a spark job from source/transformations configuration for a data product. This will result in a
@@ -75,12 +84,12 @@ def generate_job(
     def _request(ctx: typer.Context, data: typing.Dict) -> httpx.Response:
         return util.post(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/generate",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/generate",
             json=data,
         )
 
     fp = util.get_file_location(json_filepath)
-    data = util.load_json_file(fp, "builder")
+    data = util.load_object_file(fp, "builder")
     data["preview"] = dry_run
     data["reassign"] = reassign
 
@@ -92,15 +101,19 @@ def generate_job(
 @app.command()
 def fetch_job_preview(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Fetch output of generate dry-run."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/preview",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/preview",
         )
 
     r = _request(ctx)
@@ -111,15 +124,19 @@ def fetch_job_preview(
 @app.command()
 def job_history(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get history of spark applications."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/history",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/history",
         )
 
     r = _request(ctx)
@@ -130,9 +147,13 @@ def job_history(
 @app.command()
 def run_history(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    suffix: str = typer.Option(None, "--suffix", "-s"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    suffix: str = typer.Option(None, "--suffix", "-s", callback=util.sanitize),
+) -> None:
     """Get history of spark application runs."""
 
     @ensure_login
@@ -140,7 +161,7 @@ def run_history(
         return util.get(
             ctx,
             "{spark_url}/history/{suffix}/run".format(
-                spark_url=spark_url(product_name, ctx.obj.gateway_api_url),
+                spark_url=_spark_url(product_name, ctx.obj.gateway_api_url),
                 suffix=suffix,
             ),
         )
@@ -153,10 +174,14 @@ def run_history(
 @app.command()
 def job_status(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    suffix: str = typer.Option(None, "--suffix", "-s"),
-    run: str = typer.Option(None, "--run", "-r"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    suffix: str = typer.Option(None, "--suffix", "-s", callback=util.sanitize),
+    run: str = typer.Option(None, "--run", "-r", callback=util.sanitize),
+) -> None:
     """Get status of a spark job.
 
     Defaults to current application status, previous application status or a
@@ -175,7 +200,7 @@ def job_status(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}",
             params=params,
         )
 
@@ -184,17 +209,23 @@ def job_status(
     process_response(r)
 
 
-def render_logs(payload: typing.Dict):
-    return "\n".join(payload["logs"])
+def _render_logs(payload: typing.Union[typing.Dict, typing.List]) -> str:
+    """Render response logs as a string for display in terminal."""
+    logs = payload["logs"] if isinstance(payload, dict) else payload
+    return "\n".join(logs)
 
 
 @app.command()
 def job_logs(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    suffix: str = typer.Option(None, "--suffix", "-s"),
-    run: str = typer.Option(None, "--run", "-r"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    suffix: str = typer.Option(None, "--suffix", "-s", callback=util.sanitize),
+    run: str = typer.Option(None, "--run", "-r", callback=util.sanitize),
+) -> None:
     """Get logs for a spark job.
 
     Defaults to current application logs, previous application logs or a
@@ -213,21 +244,25 @@ def job_logs(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/log",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/log",
             params=params,
         )
 
     r = _request(ctx)
 
-    process_response(r, render_logs)
+    process_response(r, _render_logs)
 
 
 @app.command()
 def update_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    job_filepath: str = typer.Option(None, "--job-filepath", "-f"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    job_filepath: str = typer.Option(None, "--job-filepath", "-f", callback=util.sanitize),
+) -> None:
     """Update an assigned spark job.
 
     Update the assigned spark job file and/or the spark job configuration values.
@@ -242,7 +277,7 @@ def update_job(
 
         return util.put(
             ctx,
-            spark_url(product_name, ctx.obj.gateway_api_url),
+            _spark_url(product_name, ctx.obj.gateway_api_url),
             files=files,
         )
 
@@ -260,20 +295,25 @@ def update_job(
 @app.command()
 def remove_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    *,
     force: bool = typer.Option(
         False,
         "--force",
         help="Force remove even if application is still running.",
     ),
-):
+) -> None:
     """Remove assigned spark job."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            spark_url(product_name, ctx.obj.gateway_api_url),
+            _spark_url(product_name, ctx.obj.gateway_api_url),
             params={"force": force},
         )
 
@@ -284,8 +324,12 @@ def remove_job(
 @app.command()
 def trigger_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Trigger assigned spark job.
 
     Trigger an additional run of a spark job.
@@ -295,7 +339,7 @@ def trigger_job(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/trigger",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/trigger",
         )
 
     r = _request(ctx)
@@ -305,17 +349,28 @@ def trigger_job(
 @app.command()
 def csv_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    delimiter: str = typer.Option(",", "--delimiter", "-d", help="csv delimiter"),
-    quotechar: typing.Optional[str] = typer.Option(None, "--quote-char", "-q", help="csv quote char"),
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    delimiter: str = typer.Option(",", "--delimiter", "-d", help="csv delimiter", callback=util.sanitize),
+    quotechar: typing.Optional[str] = typer.Option(
+        None,
+        "--quote-char",
+        "-q",
+        help="csv quote char",
+        callback=util.sanitize,
+    ),
     csv_write_mode: typing.Optional[str] = typer.Option(
         None,
         "--write-mode",
         "-w",
         help="csv write mode [overwrite|append]",
+        callback=util.sanitize,
     ),
-    job_filepath: str = typer.Option(..., "--job-filepath", "-f"),
-):
+    job_filepath: str = typer.Option(..., "--job-filepath", "-f", callback=util.sanitize),
+) -> None:
     """Ingest a csv file into a data product.
 
     Upload a csv file for processing into a data product.
@@ -335,7 +390,7 @@ def csv_job(
 
         return util.post(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/csv",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/csv",
             params=params,
             files={"csv_file": f},
         )
@@ -351,9 +406,19 @@ def csv_job(
 @app.command()
 def schedule_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    schedule: str = typer.Option(..., "--schedule", "-s", help='Schedule in crontab format (e.g. "* * * * *")'),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    schedule: str = typer.Option(
+        ...,
+        "--schedule",
+        "-s",
+        help='Schedule in crontab format (e.g. "* * * * *")',
+        callback=util.sanitize,
+    ),
+) -> None:
     """Schedule an assigned spark job.
 
     Schedule a spark job once it is configured correctly to run periodically.
@@ -363,7 +428,7 @@ def schedule_job(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.post(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
             json={
                 "cron_expression": schedule,
             },
@@ -376,9 +441,19 @@ def schedule_job(
 @app.command()
 def reschedule_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    schedule: str = typer.Option(..., "--schedule", "-s", help='Schedule in crontab format (e.g. "* * * * *")'),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    schedule: str = typer.Option(
+        ...,
+        "--schedule",
+        "-s",
+        help='Schedule in crontab format (e.g. "* * * * *")',
+        callback=util.sanitize,
+    ),
+) -> None:
     """Reschedule an assigned spark job.
 
     Update existing scheduled spark job run schedule.
@@ -388,7 +463,7 @@ def reschedule_job(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.put(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
             json={
                 "cron_expression": schedule,
             },
@@ -401,8 +476,12 @@ def reschedule_job(
 @app.command()
 def unschedule_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Unschedule a scheduled spark job.
 
     Remove existing scheduled spark job.
@@ -412,7 +491,7 @@ def unschedule_job(
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/scheduled",
         )
 
     r = _request(ctx)
@@ -422,14 +501,18 @@ def unschedule_job(
 @app.command()
 def stream_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
     trigger_interval: str = typer.Option(
         ...,
         "--trigger",
         "-t",
         help='Trigger interval in spark format (e.g. "30 seconds")',
     ),
-):
+) -> None:
     """Run an infinite spark job.
 
     For streaming data product spark job will be created with given trigger interval.
@@ -441,7 +524,7 @@ def stream_job(
 
         return util.post(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
             params=params,
         )
 
@@ -452,14 +535,18 @@ def stream_job(
 @app.command()
 def restream_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
     trigger_interval: str = typer.Option(
         ...,
         "--trigger",
         "-t",
         help='Trigger interval in spark format (e.g. "30 seconds")',
     ),
-):
+) -> None:
     """Update an infinite spark job.
 
     For streaming data product spark job will be removed and recreated with given trigger interval.
@@ -471,7 +558,7 @@ def restream_job(
 
         return util.put(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
             params=params,
         )
 
@@ -482,8 +569,12 @@ def restream_job(
 @app.command()
 def unstream_job(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Pause an infinite spark job.
 
     For streaming data product spark job will be removed and recreated with trigger interval "once".
@@ -491,10 +582,9 @@ def unstream_job(
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
-
         return util.delete(
             ctx,
-            f"{spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
+            f"{_spark_url(product_name, ctx.obj.gateway_api_url)}/streaming",
         )
 
     r = _request(ctx)
@@ -506,14 +596,14 @@ def add(
     ctx: typer.Context,
     product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
     secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
-):
+) -> None:
     """Add a set of secrets for a spark job."""
 
     @ensure_login
     def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
         return util.post(
             ctx,
-            secret_url(product_name, ctx.obj.gateway_api_url),
+            _secret_url(product_name, ctx.obj.gateway_api_url),
             json=payload,
         )
 
@@ -532,7 +622,7 @@ def update(
     ctx: typer.Context,
     product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
     secrets: typing.List[str] = typer.Option(..., "--secret", "-s", help="Secret in the form key:value"),
-):
+) -> None:
     """Update existing secrets.
 
     This will overwrite existing keys, and add new keys, any keys that already
@@ -543,7 +633,7 @@ def update(
     def _request(ctx: typer.Context, payload: typing.Dict) -> httpx.Response:
         return util.patch(
             ctx,
-            secret_url(product_name, ctx.obj.gateway_api_url),
+            _secret_url(product_name, ctx.obj.gateway_api_url),
             json=payload,
         )
 
@@ -560,15 +650,19 @@ def update(
 @secret_app.command()
 def remove(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Remove secret."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            secret_url(product_name, ctx.obj.gateway_api_url),
+            _secret_url(product_name, ctx.obj.gateway_api_url),
         )
 
     r = _request(ctx)
@@ -579,16 +673,26 @@ def remove(
 @secret_app.command()
 def remove_key(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-    keys: typing.List[str] = typer.Option(..., "--key", "-k", help="Key name you wish to remove from secret"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+    keys: typing.List[str] = typer.Option(
+        ...,
+        "--key",
+        "-k",
+        help="Key name you wish to remove from secret",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Remove a set of keys from a secret."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.delete(
             ctx,
-            f"{secret_url(product_name, ctx.obj.gateway_api_url)}/key",
+            f"{_secret_url(product_name, ctx.obj.gateway_api_url)}/key",
             json={"keys": keys},
         )
 
@@ -600,15 +704,19 @@ def remove_key(
 @secret_app.command()
 def get(
     ctx: typer.Context,
-    product_name: str = typer.Argument(os.getenv("NEOSCTL_PRODUCT", ...), help="Data Product name"),
-):
+    product_name: str = typer.Argument(
+        os.getenv("NEOSCTL_PRODUCT", ...),
+        help="Data Product name",
+        callback=util.sanitize,
+    ),
+) -> None:
     """Get existing secret keys."""
 
     @ensure_login
     def _request(ctx: typer.Context) -> httpx.Response:
         return util.get(
             ctx,
-            secret_url(product_name, ctx.obj.gateway_api_url),
+            _secret_url(product_name, ctx.obj.gateway_api_url),
         )
 
     r = _request(ctx)
